@@ -21,9 +21,10 @@ import java.util.Locale;
  *
  * @author Mike McCabe
  *     <p>Significant parts of this code are adapted from the venerable jsdate.cpp (also Mozilla):
- *     https://dxr.mozilla.org/mozilla-central/source/js/src/jsdate.cpp
+ *     <a href="https://dxr.mozilla.org/mozilla-central/source/js/src/jsdate.cpp">jsdate.cpp</a>
  */
 @SuppressWarnings("AndroidJdkLibsChecker")
+// java.time.format API added in API level 26
 final class NativeDate extends IdScriptableObject {
     private static final long serialVersionUID = -8307438915861678966L;
 
@@ -953,8 +954,8 @@ final class NativeDate extends IdScriptableObject {
      * Parse input string according to simplified ISO-8601 Extended Format:
      *
      * <ul>
-     *   <li><code>YYYY-MM-DD'T'HH:mm:ss.sss'Z'</code>
-     *   <li>or <code>YYYY-MM-DD'T'HH:mm:ss.sss[+-]hh:mm</code>
+     *   <li>{@code YYYY-MM-DD'T'HH:mm:ss.sss'Z'}
+     *   <li>or {@code YYYY-MM-DD'T'HH:mm:ss.sss[+-]hh:mm}
      * </ul>
      */
     private static double parseISOString(Context cx, String s) {
@@ -976,7 +977,7 @@ final class NativeDate extends IdScriptableObject {
                 i += 1;
                 yearlen = 6;
                 yearmod = (c == '-') ? -1 : 1;
-            } else if (c == 'T') {
+            } else if (c == 'T' && cx.getLanguageVersion() < Context.VERSION_ES6) {
                 // time-only forms no longer in spec, but follow spidermonkey here
                 i += 1;
                 state = HOUR;
@@ -984,31 +985,61 @@ final class NativeDate extends IdScriptableObject {
         }
         loop:
         while (state != ERROR) {
-            int m = i + (state == YEAR ? yearlen : state == MSEC ? 3 : 2);
-            if (m > len) {
-                state = ERROR;
-                break;
-            }
-
-            int value = 0;
-            for (; i < m; ++i) {
-                char c = s.charAt(i);
-                if (c < '0' || c > '9') {
+            if (state == MSEC) {
+                // milli secs are different, digit 2 and 3 are optional
+                int value = 0;
+                int digitsFound = 0;
+                for (; i < len; i++) {
+                    char c = s.charAt(i);
+                    if (c < '0' || c > '9') {
+                        break;
+                    }
+                    if (digitsFound < 3) {
+                        value = 10 * value + (c - '0');
+                        digitsFound++;
+                    }
+                    // Simply ignore any digits beyond the first 3
+                }
+                if (digitsFound == 0) {
                     state = ERROR;
                     break loop;
                 }
-                value = 10 * value + (c - '0');
-            }
-            values[state] = value;
+                if (digitsFound < 3) {
+                    value = value * (digitsFound == 1 ? 100 : 10);
+                }
+                values[state] = value;
 
-            if (i == len) {
-                // reached EOF, check for end state
-                switch (state) {
-                    case HOUR:
-                    case TZHOUR:
+                if (i == len) {
+                    // no timezone at all is correct here
+                    break;
+                }
+            } else {
+                int m = i + (state == YEAR ? yearlen : 2);
+                if (m > len) {
+                    state = ERROR;
+                    break;
+                }
+
+                int value = 0;
+                for (; i < m; ++i) {
+                    char c = s.charAt(i);
+                    if (c < '0' || c > '9') {
                         state = ERROR;
+                        break loop;
+                    }
+                    value = 10 * value + (c - '0');
+                }
+                values[state] = value;
+
+                if (i == len) {
+                    // reached EOF, check for end state
+                    switch (state) {
+                        case HOUR:
+                        case TZHOUR:
+                            state = ERROR;
+                    }
+                    break;
                 }
-                break;
             }
 
             char c = s.charAt(i++);
@@ -1017,6 +1048,9 @@ final class NativeDate extends IdScriptableObject {
                 values[TZHOUR] = 0;
                 values[TZMIN] = 0;
                 switch (state) {
+                    case YEAR:
+                    case MONTH:
+                    case DAY:
                     case MIN:
                     case SEC:
                     case MSEC:
