@@ -16,6 +16,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import org.mozilla.javascript.AbstractEcmaStringOperations.ReplacementOperation;
 import org.mozilla.javascript.ScriptRuntime.StringIdOrIndex;
 
 /**
@@ -910,8 +911,14 @@ final class NativeString extends ScriptableObject {
         String string = ScriptRuntime.toString(o);
         String searchString = ScriptRuntime.toString(searchValue);
         boolean functionalReplace = replaceValue instanceof Callable;
+        List<ReplacementOperation> replaceOps;
+
         if (!functionalReplace) {
-            replaceValue = ScriptRuntime.toString(replaceValue);
+            replaceOps =
+                    AbstractEcmaStringOperations.buildReplacementList(
+                            ScriptRuntime.toString(replaceValue));
+        } else {
+            replaceOps = List.of();
         }
         int searchLength = searchString.length();
         int position = string.indexOf(searchString);
@@ -937,7 +944,7 @@ final class NativeString extends ScriptableObject {
                                     });
             replacement = ScriptRuntime.toString(replacementObj);
         } else {
-            NativeArray captures = (NativeArray) cx.newArray(scope, 0);
+            List<Object> captures = List.of();
             replacement =
                     AbstractEcmaStringOperations.getSubstitution(
                             cx,
@@ -947,7 +954,7 @@ final class NativeString extends ScriptableObject {
                             position,
                             captures,
                             Undefined.SCRIPTABLE_UNDEFINED,
-                            (String) replaceValue);
+                            replaceOps);
         }
         return preceding + replacement + following;
     }
@@ -992,8 +999,13 @@ final class NativeString extends ScriptableObject {
         String string = ScriptRuntime.toString(o);
         String searchString = ScriptRuntime.toString(searchValue);
         boolean functionalReplace = replaceValue instanceof Callable;
+        List<ReplacementOperation> replaceOps;
         if (!functionalReplace) {
-            replaceValue = ScriptRuntime.toString(replaceValue);
+            replaceOps =
+                    AbstractEcmaStringOperations.buildReplacementList(
+                            ScriptRuntime.toString(replaceValue));
+        } else {
+            replaceOps = List.of();
         }
         int searchLength = searchString.length();
         int advanceBy = Math.max(1, searchLength);
@@ -1029,7 +1041,7 @@ final class NativeString extends ScriptableObject {
                                         });
                 replacement = ScriptRuntime.toString(replacementObj);
             } else {
-                NativeArray captures = (NativeArray) cx.newArray(scope, 0);
+                List<Object> captures = List.of();
                 replacement =
                         AbstractEcmaStringOperations.getSubstitution(
                                 cx,
@@ -1039,7 +1051,7 @@ final class NativeString extends ScriptableObject {
                                 p,
                                 captures,
                                 Undefined.SCRIPTABLE_UNDEFINED,
-                                (String) replaceValue);
+                                replaceOps);
             }
             result.append(preserved);
             result.append(replacement);
