@@ -8,7 +8,7 @@ import static org.junit.jupiter.api.Assertions.fail;
 import java.util.concurrent.atomic.AtomicBoolean;
 import org.junit.jupiter.api.Nested;
 import org.junit.jupiter.api.Test;
-import org.mozilla.javascript.tests.Utils;
+import org.mozilla.javascript.testutils.Utils;
 
 class SuperTest {
     @Nested
@@ -345,16 +345,7 @@ class SuperTest {
 
         @Test
         void getPropMissingWithContextFeatureStrictMode() {
-            ContextFactory factory =
-                    new ContextFactory() {
-                        @Override
-                        protected boolean hasFeature(Context cx, int featureIndex) {
-                            if (featureIndex == Context.FEATURE_STRICT_MODE) {
-                                return true;
-                            }
-                            return super.hasFeature(cx, featureIndex);
-                        }
-                    };
+            ContextFactory factory = Utils.contextFactoryWithFeatures(Context.FEATURE_STRICT_MODE);
 
             Utils.runWithAllModes(
                     factory,
@@ -1141,6 +1132,38 @@ class SuperTest {
 
             Utils.assertWithAllModes_ES6("object", script);
         }
+
+        @Test
+        void mixedCompileInterpret() {
+            String script =
+                    ""
+                            + "var proto = {\n"
+                            + "  x: 'proto',\n"
+                            + "  f() {\n"
+                            + "    return this.x;\n"
+                            + "  }\n"
+                            + "};\n"
+                            + "var object = {\n"
+                            + "   x: 'object',\n"
+                            + "   f() {\n"
+                            + "    return () => { return super.f(); };\n"
+                            + "  }\n"
+                            + "};\n"
+                            + "Object.setPrototypeOf(object, proto);\n";
+            String script2 = "object.f()();";
+
+            try (Context cx = Context.enter()) {
+                cx.setLanguageVersion(Context.VERSION_ES6);
+                ScriptableObject scope = cx.initStandardObjects();
+
+                cx.setInterpretedMode(true);
+                cx.evaluateString(scope, script, "test", 1, null);
+
+                cx.setInterpretedMode(false);
+                Object res = cx.evaluateString(scope, script2, "test", 1, null);
+                assertEquals("object", res);
+            }
+        }
     }
 
     /**
