@@ -3,7 +3,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = 'regress-280844-2.js';
+gTestfile = 'regress-280844-1.js';
 
 var summary = 'Uncontrolled recursion in js_MarkXML during GC';
 var BUGNUMBER = 280844;
@@ -13,26 +13,14 @@ var expect = 'No Crash';
 printBugNumber(BUGNUMBER);
 START(summary);
 
-var N = 100 * 1000;
-
-function prepare_list(N)
-{
-    var head = {};
-    var cursor = head;
-    for (var i = 0; i != N; ++i) {
-        var ns = new Namespace(); 
-        var x = <xml/>;
-        x.addNamespace(ns);
-        cursor.next = x;
-        cursor = ns;
-    }
-    return head;
+var N = 5 * 1000;
+var x = <x/>;
+for (var i = 1; i <= N; ++i) {
+    x.appendChild(<x/>);
+    x = x.x[0];
 }
-
-var head = prepare_list(N);
-
+printStatus(x.toXMLString());
 gc();
 
 TEST(1, expect, actual);
-
 END();
