@@ -4,28 +4,23 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = 'regress-458679-01.js';
+gTestfile = 'regress-327691-02.js';
 
-var summary = 'GetXMLEntity should not assume FastAppendChar is infallible';
-var BUGNUMBER = 458679;
-var actual = '';
-var expect = '';
+var summary = "Do not crash during gc()";
+var BUGNUMBER = 327691;
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-try
-{
-    var x = "<";
+var A=<y/>;
+var B=A.h;
+var D=B.h;
+B.h=D.h;
+B[0]=B.h;
 
-    while (x.length < 12000000)
-        x += x;
-
-    <e4x>{x}</e4x>;
-}
-catch(ex)
-{
-}
+gc();
 
 TEST(1, expect, actual);
 
