@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = '7.3-11.js';
+gTestfile = '7.3-3.js';
 
 /**
-   File Name:          7.3-11.js
+   File Name:          7.3-3.js
    ECMA Section:       7.3 Comments
    Description:
 
@@ -15,19 +15,18 @@ gTestfile = '7.3-11.js';
    Date:               12 november 1997
 
 */
-var SECTION = "7.3-11";
+var SECTION = "7.3-3";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Comments";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-
 var testcase =  new TestCase( SECTION,
-			      "code following multiline comment",
+			      "source text directly following a single-line comment",
 			      "pass",
-			      "pass");
-
-////testcase.actual="fail";
+			      "fail");
+// a comment string
+testcase.actual = "pass";
 
 test();
