@@ -3,13 +3,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = '12.6.2-3.js';
+gTestfile = '12.6.2-8.js';
 
 /**
-   File Name:          12.6.2-3.js
+   File Name:          12.6.2-8.js
    ECMA Section:       12.6.2 The for Statement
 
-   1. first expression is not present.
+   1. first expression is present.
    2. second expression is present
    3. third expression is present
 
@@ -17,22 +17,21 @@ gTestfile = '12.6.2-3.js';
    Author:             christine@netscape.com
    Date:               15 september 1997
 */
-var SECTION = "12.6.2-3";
+var SECTION = "12.6.2-8";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "The for statement";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( SECTION, "for statement",  100,     testprogram() );
+new TestCase( SECTION, "for statement",  256,     testprogram() );
 
 test();
 
 function testprogram() {
-  myVar = 0;
+  var myVar;
 
-  for ( ; myVar < 100 ; myVar++ ) {
-    continue;
+  for ( myVar=2;  myVar < 256; myVar *= myVar ) {
   }
 
   return myVar;
