@@ -6,15 +6,15 @@
 /*
  *
  * Date:    14 April 2003
- * SUMMARY: Prototype of predefined error objects should be DontDelete
+ * SUMMARY: Prototype of predefined error objects should be DontEnum
  * See http://bugzilla.mozilla.org/show_bug.cgi?id=201989
  *
  */
 //-----------------------------------------------------------------------------
-var gTestfile = '15.11.7.6-002.js';
+var gTestfile = '15.11.7.6-001.js';
 var UBound = 0;
 var BUGNUMBER = 201989;
-var summary = 'Prototype of predefined error objects should be DontDelete';
+var summary = 'Prototype of predefined error objects should be DontEnum';
 var status = '';
 var statusitems = [];
 var actual = '';
@@ -24,20 +24,18 @@ var expectedvalues = [];
 
 
 /*
- * Tests that |F.prototype| is DontDelete
+ * Tests that |F.prototype| is not enumerable in |F|
  */
-function testDontDelete(F)
+function testDontEnum(F)
 {
-  var e;
-  var orig = F.prototype;
-  try
+  var proto = F.prototype;
+
+  for (var prop in F)
   {
-    delete F.prototype;
+    if (F[prop] === proto)
+      return false;
   }
-  catch (e)
-  {
-  }
-  return F.prototype === orig;
+  return true;
 }
 
 
@@ -60,8 +58,8 @@ for (i in list)
   // Test for |F|; e.g. Rhino defines |ConversionError| while SM does not.
   if (F)
   {
-    status = 'Testing DontDelete attribute of |' + list[i] + '.prototype|';
-    actual = testDontDelete(F);
+    status = 'Testing DontEnum attribute of |' + list[i] + '.prototype|';
+    actual = testDontEnum(F);
     expect = true;
     addThis();
   }
