@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = '11.7.3-01.js';
+var gTestfile = '11.7.2-01.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 396969;
-var summary = '11.7.3 - >>> should evaluate operands in order';
+var summary = '11.7.2 - >> should evaluate operands in order';
 var actual = '';
 var expect = '';
 
@@ -35,7 +35,7 @@ function test()
     value:2
   };
 
-  actualval = (o >>> p);
+  actualval = (o >> p);
 
   reportCompare(expectval, actualval, summary + ': value');
   reportCompare(expect, actual, summary + ': order');
