@@ -3,27 +3,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-460886-02.js';
+var gTestfile = 'regress-317714-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 460886;
-var summary = 'Do not crash @ js_NewStringCopy';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 317714;
+var summary = 'Regression test for regression from bug 316885';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
+var d5="-1";
+var r3=d5.split(":");
+r3[0]++;
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-function test()
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
-
-  for (var j = 0; j < 5; ++j) { "".substring(-60000); } 
- 
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
-}
+reportCompare(expect, actual, summary);
